#
# $Id: Build.PL,v 5379eba6a825 2015/10/27 19:07:21 gomor $
#
use strict;
use warnings;

use Module::Build;

my $builder = Module::Build->new(
   module_name => 'Metabrik::Repository',
   license => 'bsd',
   dist_author => 'GomoR <gomor_at_metabrik.org>',
   dist_version_from => 'lib/Metabrik/Repository.pm',
   requires => {
      'perl' => '5.6.1',
      'Metabrik' => '1.10',
      'DBD::SQLite' => 0,
      'DBI' => 0,
      'Data::Dump' => 0,
      'Data::Dumper' => 0,
      'File::Copy' => 0,
      'File::Find' => 0,
      'File::Spec' => 0,
      'IO::All' => 0,
      'IO::Handle' => 0,
      'IO::Scalar' => 0,
      'IO::Select' => 0,
      'IO::Socket::INET' => 0,
      'IO::Socket::INET6' => 0,
      'IO::Socket::Multicast' => 0,
      'IO::Socket::SSL' => 0,
      'IPC::Run3' => 0,
      'JSON::XS' => 0,
      'LWP::UserAgent' => 0,
      'List::Util' => 0,
      'MIME::Base64' => 0,
      'Net::CIDR' => 0,
      'Net::DNS' => 0,
      'Net::Frame' => 0,
      'Net::Frame::Dump' => 0,
      'Net::Frame::Layer::ICMPv4' => 0,
      'Net::Frame::Layer::ICMPv6' => 0,
      'Net::Frame::Layer::IPv6' => 0,
      'Net::Frame::Simple' => 0,
      'Net::IPv4Addr' => 0,
      'Net::IPv6Addr' => 0,
      'Net::Libdnet' => 0,
      'Net::Netmask' => 0,
      'Net::Nslookup' => 0,
      'Net::Pcap' => 0,
      'Net::Routing' => 0,
      'Net::SSL' => 0,
      'Net::SSLeay' => 0,
      'Net::Write' => 0,
      'Net::Write::Fast' => 0,
      'NetAddr::IP' => 0,
      'POSIX' => 0,
      'Storable' => 0,
      'String::Random' => 0,
      'Term::ANSIColor' => 0,
      'Term::ReadPassword' => 0,
      'Text::CSV_XS' => 0,
      'URI' => 0,
      'URI::Escape' => 0,
      'WWW::Mechanize' => 0,
      'XML::Simple' => 0,
   },
);

$builder->create_build_script;
